//
// Copyright (c) 2016, MindFusion LLC - Bulgaria.
//

using System;
using System.Collections;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.IO;
using System.Text;
using System.Windows.Forms;
using System.Xml;

using System.Runtime.InteropServices;

using MindFusion.Scheduling;
using MindFusion.Scheduling.WinForms;

using ImageAlignment = MindFusion.Drawing.ImageAlignment;


namespace MindFusion.Scheduling.WinForms.Samples.CS.AdvancedItemDrawing
{
	public partial class MainForm : Form
	{
		public MainForm()
		{
			InitializeComponent();

			_appPath = Path.GetDirectoryName(Application.ExecutablePath);

			_lbInfo.DocumentText = Resources.Info;
			_lbInfo.DocumentCompleted += new WebBrowserDocumentCompletedEventHandler(OnDocumentCompleted);

			calendar.ItemSettings.ShadowStyle = ShadowStyle.None;
			calendar.ItemSettings.UseExtendedStyles = State.Disabled;
			calendar.EnableOptimization = true;

			// Ensure we are createing items of the appropriate type
			calendar.InteractiveItemType = typeof(CustomAppointment);

			// Register our class for serialization
			Schedule.RegisterItemClass(typeof(CustomAppointment), "roundapp", 1);

			// Notify for initial theme
			ThemeChanged();

			// Create some initial items
			CustomAppointment app = null;

			app = new CustomAppointment();
			app.StartTime = calendar.Date.Date + TimeSpan.FromHours(2.5);
			app.EndTime = calendar.Date.Date + TimeSpan.FromHours(7);
			app.HeaderText = "Custom Appointment";
			app.DescriptionText = "<b>MindFusion.Scheduling</b> is a software component that can be used to present calendars and timetables to users and let them edit the schedule information interactively. It is implemented as a .NET Windows Forms control and can be easily integrated into any application targeting the <i>Microsoft</i> .NET platform. By providing your software with ready to use capabilities for visualizing and editing schedule data, <b>MindFusion.Scheduling</b> saves you from hundreds of hours of coding and debugging efforts.";
			app.HeaderImages.Add(Resources.Icon1);
			app.HeaderImages.Add(Resources.Icon3);
			app.HeaderImages.Add(Resources.Icon4);
			app.ContentsImage = Resources.CarImage;
			app.ContentsImageAlignment = ImageAlignment.Left | ImageAlignment.Top;
			calendar.Schedule.Items.Add(app);

			app = new CustomAppointment();
			app.StartTime = calendar.Date.Date + TimeSpan.FromHours(6);
			app.EndTime = calendar.Date.Date + TimeSpan.FromHours(8.5);
			app.HeaderText = "Month Range View";
			app.DescriptionText = "Set the <u><color=#0000ff>CurrentView</color></u> property of the <u><color=#0000ff>Calendar</color></u> class to MonthRange to select the month range view. This view displays a range of months or years. Each month represents a grid of day cells, which display the day number and the schedule items set to occur on the specific date. The month in the view can be scrolled back or forth in time by an arbitrary interval using the navigation buttons in the view header.";
			app.HeaderImages.Add(Resources.Icon2);
			app.HeaderImages.Add(Resources.Icon3);
			app.ContentsImage = Resources.CarImage;
			app.ContentsImageAlignment = ImageAlignment.Right | ImageAlignment.Top;
			calendar.Schedule.Items.Add(app);

			app = new CustomAppointment();
			app.StartTime = calendar.Date.Date + TimeSpan.FromHours(3);
			app.EndTime = calendar.Date.Date + TimeSpan.FromHours(5.5);
			app.HeaderText = "Custom Themes";
			app.DescriptionText = "Since version 4.2 <b>MindFusion.Scheduling</b> enables the creation of user-defined themes. The <u><color=#0000ff>Theme</color></u> class is now abstract and can be inherited. In order to create a custom theme, you have to define a new class derived from <u><color=#990099>Theme</color></u> and implement all abstract properties.";
			app.HeaderImages.Add(Resources.Icon1);
			app.HeaderImages.Add(Resources.Icon3);
			app.HeaderImages.Add(Resources.Icon3);
			app.HeaderImages.Add(Resources.Icon5);
			app.ContentsImage = Resources.CarImage;
			app.ContentsImageAlignment = ImageAlignment.Right | ImageAlignment.Top;
			calendar.Schedule.Items.Add(app);

			app = new CustomAppointment();
			app.StartTime = calendar.Date.Date + TimeSpan.FromHours(0.5);
			app.EndTime = calendar.Date.Date + TimeSpan.FromHours(2);
			app.HeaderText = "Recurrence";
			app.DescriptionText = "Schedule items can recur in time, following a specific recurrence pattern. For example an event could be set to occur each <b><i>Monday</i></b>, on the third day of each month, and so on.";
			app.HeaderImages.Add(Resources.Icon5);
			app.ContentsImage = Resources.CarImage;
			app.ContentsImageAlignment = ImageAlignment.Unspecified;
			calendar.Schedule.Items.Add(app);

			app = new CustomAppointment();
			app.StartTime = calendar.Date.Date;
			app.EndTime = calendar.Date.Date;
			app.AllDayEvent = true;
			app.HeaderText = "Advanced Item Drawing";
			app.DescriptionText = "";
			app.HeaderImages.Add(Resources.Icon4);
			app.HeaderImages.Add(Resources.Icon5);
			app.ContentsImage = Resources.CarImage;
			app.ContentsImageAlignment = ImageAlignment.Unspecified;
			calendar.Schedule.Items.Add(app);
		}

		private void OnDocumentCompleted(object sender, WebBrowserDocumentCompletedEventArgs e)
		{
			if (_lbInfo.Document != null)
			{
				_lbInfo.Document.Click += Document_Click;
				_lbInfo.Document.Body.Style = "font-family: Verdana, Arial, Helvetica, sans-serif; " +
					"font-size: 7.5pt; margin: 2pt; padding: 0pt; background-color: #ffffee; cursor: default;";
			}
		}

		/// <summary>
		/// Handles the Web browser document's Click event.
		/// </summary>
		private void Document_Click(object sender, HtmlElementEventArgs e)
		{
			e.ReturnValue = true;

			HtmlElement pointed = _lbInfo.Document.GetElementFromPoint(e.ClientMousePosition);
			if (pointed == null)
				return;

			if (pointed.TagName.ToLower() == "a")
			{
				string href = pointed.GetAttribute("href").ToLower();
				if (href.StartsWith("help:"))
				{
					string topic = href.Substring(5);
					string helpFile = ResolveHelpPath(_appPath);

					if (System.IO.File.Exists(helpFile))
					{
						Help.ShowHelp(this, helpFile, HelpNavigator.Topic, topic);

						// Prevent link follow
						e.ReturnValue = false;
					}
				}
			}
		}

		private static string ResolveHelpPath(string baseDir)
		{
			string[] helpFiles = new string[]
			{
				"MindFusion.Scheduling.chm",
				"MindFusion WinForms.chm"
			};

			DirectoryInfo dirInfo = new DirectoryInfo(baseDir);
			int i = 0;
			while (dirInfo != null)
			{
				string helpFile = Path.Combine(dirInfo.FullName, helpFiles[i]);
				if (File.Exists(helpFile))
					return helpFile;

				i++;
				if (i == helpFiles.Length)
				{
					i = 0;
					dirInfo = dirInfo.Parent;
				}
			}

			return string.Empty;
		}

		private void OnAbout(object sender, System.EventArgs e)
		{
			System.Reflection.Assembly assembly = System.Reflection.Assembly.GetAssembly(
				typeof(MindFusion.Scheduling.WinForms.Calendar));

			if (assembly == null)
				return;

			Type aboutFormType = assembly.GetType(
				"MindFusion.Scheduling.WinForms.AboutForm");

			if (aboutFormType == null)
				return;

			((Form)Activator.CreateInstance(aboutFormType)).ShowDialog(this);
		}

		private void OnExit(object sender, System.EventArgs e)
		{
			this.Close();
		}

		private void calendar_ItemCreated(object sender, MindFusion.Scheduling.ItemEventArgs e)
		{
			// Set the transparency and radius values of the item
			if (e.Item is CustomAppointment)
			{
				CustomAppointment app = e.Item as CustomAppointment;
				app.Radius = trackRoundness.Value;
				app.Transparency = trackTransparency.Value * 10;
				app.ContentsImage = Resources.CarImage;
				app.ContentsImageAlignment = ImageAlignment.Left | ImageAlignment.Top;
			}
		}

		/// <summary>
		/// Handles the Scroll event of the 'Transparency' icon.
		/// </summary>
		private void trackTransparency_Scroll(object sender, EventArgs e)
		{
			// Set the transparency value of all items in the schedule
			foreach (CustomAppointment app in calendar.Schedule.Items)
				app.Transparency = trackTransparency.Value * 10;
		}

		/// <summary>
		/// Handles the Scroll event of the 'Roundness' icon.
		/// </summary>
		private void trackRoundness_Scroll(object sender, EventArgs e)
		{
			// Set the radius value of all items in the schedule
			foreach (CustomAppointment app in calendar.Schedule.Items)
				app.Radius = trackRoundness.Value;
		}

		/// <summary>
		/// Handles the DropDownOpening event of the 'Theme' menu.
		/// </summary>
		private void menuItemTheme_DropDownOpening(object sender, EventArgs e)
		{
			menuItemLila.Checked = calendar.Theme == ThemeType.Lila;
			menuItemStandard.Checked = calendar.Theme == ThemeType.Standard;
			menuItemSilver.Checked = calendar.Theme == ThemeType.Silver;
			menuItemWindows2003.Checked = calendar.Theme == ThemeType.Windows2003;
			menuItemVista.Checked = calendar.Theme == ThemeType.Vista;
			menuItemLight.Checked = calendar.Theme == ThemeType.Light;
		}

		/// <summary>
		/// Handles the DropDownOpening event of the 'View' menu.
		/// </summary>
		private void menuItemView_DropDownOpening(object sender, EventArgs e)
		{
			menuItemList.Checked = calendar.CurrentView == CalendarView.List;
			menuItemSingleMonth.Checked = calendar.CurrentView == CalendarView.SingleMonth;
			menuItemTimetable.Checked = calendar.CurrentView == CalendarView.Timetable;
			menuItemWeekRange.Checked = calendar.CurrentView == CalendarView.WeekRange;
		}

		/// <summary>
		/// Handles the Click event of the 'Standard' menu item.
		/// </summary>
		private void menuItemStandard_Click(object sender, EventArgs e)
		{
			calendar.Theme = ThemeType.Standard;

			ThemeChanged();
		}

		/// <summary>
		/// Handles the Click event of the 'Silver' menu item.
		/// </summary>
		private void menuItemSilver_Click(object sender, EventArgs e)
		{
			calendar.Theme = ThemeType.Silver;

			ThemeChanged();
		}

		/// <summary>
		/// Handles the Click event of the 'Lila' menu item.
		/// </summary>
		private void menuItemLila_Click(object sender, EventArgs e)
		{
			calendar.Theme = ThemeType.Lila;

			ThemeChanged();
		}

		/// <summary>
		/// Handles the Click event of the 'Windows2003' menu item.
		/// </summary>
		private void menuItemWindows2003_Click(object sender, EventArgs e)
		{
			calendar.Theme = ThemeType.Windows2003;

			ThemeChanged();
		}

		/// <summary>
		/// Handles the Click event of the 'Vista' menu item.
		/// </summary>
		private void menuItemVista_Click(object sender, EventArgs e)
		{
			calendar.Theme = ThemeType.Vista;

			ThemeChanged();
		}

		/// <summary>
		/// Handles the Click event of the 'Light' menu item.
		/// </summary>
		private void menuItemLight_Click(object sender, EventArgs e)
		{
			calendar.Theme = ThemeType.Light;

			ThemeChanged();
		}

		/// <summary>
		/// Invoked after the current theme has changed
		/// </summary>
		private void ThemeChanged()
		{
			// Make the item's header brushes semitransparent
			MindFusion.Drawing.Brush[] brushes = new MindFusion.Drawing.Brush[]
			{
				calendar.ItemSettings.Style.HeaderBrush,
				calendar.ItemSettings.SelectedItemStyle.HeaderBrush,
				calendar.ItemSettings.PointedItemStyle.HeaderBrush,
				calendar.ItemSettings.PointedSelectedItemStyle.HeaderBrush
			};
			int alpha = 220;
			foreach (MindFusion.Drawing.Brush b in brushes)
			{
				if (b is MindFusion.Drawing.LinearGradientBrush)
				{
					MindFusion.Drawing.LinearGradientBrush lb = b as MindFusion.Drawing.LinearGradientBrush;
					lb.LinearColors[0] = Color.FromArgb(alpha, lb.LinearColors[0]);
					lb.LinearColors[1] = Color.FromArgb(alpha, lb.LinearColors[1]);
					if (lb.InterpolationColors != null)
					{
						for (int i = 0; i < lb.InterpolationColors.Colors.Length; i++)
							lb.InterpolationColors.Colors[i] = Color.FromArgb(alpha, lb.InterpolationColors.Colors[i]);
					}

				}
				else if (b is MindFusion.Drawing.SolidBrush)
				{
					MindFusion.Drawing.SolidBrush sb = b as MindFusion.Drawing.SolidBrush;
					sb.Color = Color.FromArgb(alpha, sb.Color);
				}
			}

			calendar.Invalidate();
		}

		/// <summary>
		/// Handles the Click event of the 'SingleMonth' menu item.
		/// </summary>
		private void menuItemSingleMonth_Click(object sender, EventArgs e)
		{
			calendar.CurrentView = CalendarView.SingleMonth;
		}

		/// <summary>
		/// Handles the Click event of the 'List' menu item.
		/// </summary>
		private void menuItemList_Click(object sender, EventArgs e)
		{
			calendar.CurrentView = CalendarView.List;
		}

		/// <summary>
		/// Handles the Click event of the 'Timetable' menu item.
		/// </summary>
		private void menuItemTimetable_Click(object sender, EventArgs e)
		{
			calendar.CurrentView = CalendarView.Timetable;
		}

		/// <summary>
		/// Handles the Click event of the 'WeekRange' menu item.
		/// </summary>
		private void menuItemWeekRange_Click(object sender, EventArgs e)
		{
			calendar.CurrentView = CalendarView.WeekRange;
		}

		/// <summary>
		/// Handles the MouseDown event of all pictures representing header icons.
		/// </summary>
		private void OnIconMouseDown(object sender, MouseEventArgs e)
		{
			// Start dragging the clicked icon
			DoDragDrop(new IconDragData((sender as PictureBox).Image), DragDropEffects.Copy);
		}

		/// <summary>
		/// Handles the DragOver event of the Calendar control.
		/// </summary>
		private void calendar_DragOver(object sender, DragEventArgs e)
		{
			// Check if data is available
			if (e.Data.GetDataPresent(typeof(IconDragData)))
			{
				// Check if over an item
				Point screen = new Point(e.X, e.Y);
				Point client = calendar.PointToClient(screen);

				if (calendar.GetItemAt(client) != null)
				{
					// Enable the drag and drop operation
					e.Effect = DragDropEffects.Copy;
					return;
				}
			}

			// Prevent the drag and drop operation
			e.Effect = DragDropEffects.None;
		}

		/// <summary>
		/// Handles the DragOver event of the Calendar control.
		/// </summary>
		private void calendar_DragDrop(object sender, DragEventArgs e)
		{
			// Check if data is available
			if (e.Data.GetDataPresent(typeof(IconDragData)))
			{
				// Check if over an item
				Point screen = new Point(e.X, e.Y);
				Point client = calendar.PointToClient(screen);

				Item item = calendar.GetItemAt(client);
				if (item != null)
				{
					// Add the dragged icon to the pointed item
					IconDragData data = e.Data.GetData(typeof(IconDragData)) as IconDragData;
					CustomAppointment app = item as CustomAppointment;
					app.HeaderImages.Add(data.Image);
					app.UpdateVisuals();
				}
			}
		}

		/// <summary>
		/// Handles the ItemClick event of the Calendar control.
		/// </summary>
		private void calendar_ItemClick(object sender, ItemMouseEventArgs e)
		{
			if (e.Clicks == 2 && e.Button == MouseButtons.Left)
			{
				calendar.ResetDrag();

				AppointmentForm form = new AppointmentForm(calendar.Schedule);
				form.SetAppointment(e.Item as Appointment);
				form.ShowDialog(this);
			}
		}

		/// <summary>
		/// Handles the DropDownOpening event of the 'Contents Image' context menu.
		/// </summary>
		private void menuItemContentsImage_DropDownOpening(object sender, EventArgs e)
		{
			menuItemNone.Enabled = clickedAppointment != null;
			menuItemLeft.Enabled = clickedAppointment != null;
			menuItemTopLeft.Enabled = clickedAppointment != null;
			menuItemTop.Enabled = clickedAppointment != null;
			menuItemTopRight.Enabled = clickedAppointment != null;
			menuItemRight.Enabled = clickedAppointment != null;
			menuItemBottomRight.Enabled = clickedAppointment != null;
			menuItemBottom.Enabled = clickedAppointment != null;
			menuItemBottomLeft.Enabled = clickedAppointment != null;

			if (clickedAppointment != null)
			{
				menuItemNone.Checked = clickedAppointment.ContentsImageAlignment == ImageAlignment.Unspecified;
				menuItemLeft.Checked = clickedAppointment.ContentsImageAlignment == ImageAlignment.Left;
				menuItemTopLeft.Checked = clickedAppointment.ContentsImageAlignment == (ImageAlignment.Left | ImageAlignment.Top);
				menuItemTop.Checked = clickedAppointment.ContentsImageAlignment == ImageAlignment.Top;
				menuItemTopRight.Checked = clickedAppointment.ContentsImageAlignment == (ImageAlignment.Right | ImageAlignment.Top);
				menuItemRight.Checked = clickedAppointment.ContentsImageAlignment == ImageAlignment.Right;
				menuItemBottomRight.Checked = clickedAppointment.ContentsImageAlignment == (ImageAlignment.Right | ImageAlignment.Bottom);
				menuItemBottom.Checked = clickedAppointment.ContentsImageAlignment == ImageAlignment.Bottom;
				menuItemBottomLeft.Checked = clickedAppointment.ContentsImageAlignment == (ImageAlignment.Left | ImageAlignment.Bottom);
			}
		}

		/// <summary>
		/// Handles the Click event of the 'None' menu item.
		/// </summary>
		private void menuItemNone_Click(object sender, EventArgs e)
		{
			if (clickedAppointment != null)
				clickedAppointment.ContentsImageAlignment = ImageAlignment.Unspecified;
		}

		/// <summary>
		/// Handles the Click event of the 'Top-Left' menu item.
		/// </summary>
		private void menuItemTopLeft_Click(object sender, EventArgs e)
		{
			if (clickedAppointment != null)
				clickedAppointment.ContentsImageAlignment = ImageAlignment.Top | ImageAlignment.Left;
		}

		/// <summary>
		/// Handles the Click event of the 'Top' menu item.
		/// </summary>
		private void menuItemTop_Click(object sender, EventArgs e)
		{
			if (clickedAppointment != null)
				clickedAppointment.ContentsImageAlignment = ImageAlignment.Top;
		}

		/// <summary>
		/// Handles the Click event of the 'Top-Right' menu item.
		/// </summary>
		private void menuItemTopRight_Click(object sender, EventArgs e)
		{
			if (clickedAppointment != null)
				clickedAppointment.ContentsImageAlignment = ImageAlignment.Top | ImageAlignment.Right;
		}

		/// <summary>
		/// Handles the Click event of the 'Right' menu item.
		/// </summary>
		private void menuItemRight_Click(object sender, EventArgs e)
		{
			if (clickedAppointment != null)
				clickedAppointment.ContentsImageAlignment = ImageAlignment.Right;
		}

		/// <summary>
		/// Handles the Click event of the 'Bottom-Right' menu item.
		/// </summary>
		private void menuItemBottomRight_Click(object sender, EventArgs e)
		{
			if (clickedAppointment != null)
				clickedAppointment.ContentsImageAlignment = ImageAlignment.Bottom | ImageAlignment.Right;
		}

		/// <summary>
		/// Handles the Click event of the 'Bottom' menu item.
		/// </summary>
		private void menuItemBottom_Click(object sender, EventArgs e)
		{
			if (clickedAppointment != null)
				clickedAppointment.ContentsImageAlignment = ImageAlignment.Bottom;
		}

		/// <summary>
		/// Handles the Click event of the 'Bottom-Left' menu item.
		/// </summary>
		private void menuItemBottomLeft_Click(object sender, EventArgs e)
		{
			if (clickedAppointment != null)
				clickedAppointment.ContentsImageAlignment = ImageAlignment.Bottom | ImageAlignment.Left;
		}

		/// <summary>
		/// Handles the Click event of the 'Left' menu item.
		/// </summary>
		private void menuItemLeft_Click(object sender, EventArgs e)
		{
			if (clickedAppointment != null)
				clickedAppointment.ContentsImageAlignment = ImageAlignment.Left;
		}

		/// <summary>
		/// Handles the MouseDown event of the Calendar control.
		/// </summary>
		private void calendar_MouseDown(object sender, MouseEventArgs e)
		{
			if (e.Button == MouseButtons.Right)
			{
				// Get the clicked item (if any)
				Item item = calendar.GetItemAt(e.X, e.Y);
				clickedAppointment = item as CustomAppointment;
				if (clickedAppointment != null)
					contextMenu.Show(calendar, e.X, e.Y);
			}
		}


		private string _appPath;
		private CustomAppointment clickedAppointment;
	}
}
